#include <spi.h>

//#define DEBUG
#define BAUD 9600L

#define PIN_LED 2
#define PIN_BTN_STOP 7
#define PIN_BTN_FORWARD 9
#define PIN_BTN_BACKWARD 5
#define PIN_BTN_LEFT 6
#define PIN_BTN_RIGHT 8
#define PIN_BTN_UP 3
#define PIN_BTN_DOWN 4

#define STOP 0
#define FORWARD 1
#define BACKWARD 2
#define LEFT 3
#define RIGHT 4
#define UP 5
#define DOWN 6

#define NUM_OF_BUTTONS 7
#define DELAY_BETWEEN_LONG_PRESS 250
#define LED_FLASH_TIME 100

unsigned long buttonDelays[NUM_OF_BUTTONS];
unsigned long ledOnTime;

void setup() {
  Serial.begin(BAUD);
  pinMode(PIN_LED, OUTPUT);
  pinMode(PIN_BTN_STOP, INPUT);
  pinMode(PIN_BTN_FORWARD, INPUT);
  pinMode(PIN_BTN_BACKWARD, INPUT);
  pinMode(PIN_BTN_LEFT, INPUT);
  pinMode(PIN_BTN_RIGHT, INPUT);
  pinMode(PIN_BTN_UP, INPUT);
  pinMode(PIN_BTN_DOWN, INPUT);
  
  // turn on the internal pull up resistors
  digitalWrite(PIN_BTN_STOP, HIGH);
  digitalWrite(PIN_BTN_FORWARD, HIGH);
  digitalWrite(PIN_BTN_BACKWARD, HIGH);
  digitalWrite(PIN_BTN_LEFT, HIGH);
  digitalWrite(PIN_BTN_RIGHT, HIGH);
  digitalWrite(PIN_BTN_UP, HIGH);
  digitalWrite(PIN_BTN_DOWN, HIGH);  
  
  // default values
  ledOnTime = 0;
  
  SPI_initMasterIntr();
  Serial.println("Initialized controller as SPI MASTER");
}


void loop() {
  if (ledOnTime != 0 && ledOnTime < millis()) {
    digitalWrite(PIN_LED, LOW);
    ledOnTime = 0;
  }
  
  if (digitalRead(PIN_BTN_STOP) == LOW && buttonDelays[STOP] < millis()) {
    #ifdef DEBUG
      Serial.println("stop");
    #endif
    
    buttonDelays[STOP] = millis() + DELAY_BETWEEN_LONG_PRESS;
    digitalWrite(PIN_LED, HIGH);
    ledOnTime = millis() + LED_FLASH_TIME;
    SPI_sendIntr((char *)"stop");
  }
  
  if (digitalRead(PIN_BTN_FORWARD) == LOW && buttonDelays[FORWARD] < millis()) {
    #ifdef DEBUG
      Serial.println("forward");
    #endif
    
    buttonDelays[FORWARD] = millis() + DELAY_BETWEEN_LONG_PRESS;
    digitalWrite(PIN_LED, HIGH);
    ledOnTime = millis() + LED_FLASH_TIME;
    SPI_sendIntr((char *)"forward");
  }
  
  if (digitalRead(PIN_BTN_BACKWARD) == LOW && buttonDelays[BACKWARD] < millis()) {
    #ifdef DEBUG
      Serial.println("backward");
    #endif
    
    buttonDelays[BACKWARD] = millis() + DELAY_BETWEEN_LONG_PRESS;
    digitalWrite(PIN_LED, HIGH);
    ledOnTime = millis() + LED_FLASH_TIME;
    SPI_sendIntr((char *)"backward");
  }
  
  if (digitalRead(PIN_BTN_LEFT) == LOW && buttonDelays[LEFT] < millis()) {
    #ifdef DEBUG
      Serial.println("left");
    #endif
    
    buttonDelays[LEFT] = millis() + DELAY_BETWEEN_LONG_PRESS;
    digitalWrite(PIN_LED, HIGH);
    ledOnTime = millis() + LED_FLASH_TIME;
    SPI_sendIntr((char *)"left");
  }
  
  if (digitalRead(PIN_BTN_RIGHT) == LOW && buttonDelays[RIGHT] < millis()) {
    #ifdef DEBUG
      Serial.println("right");
    #endif
    
    buttonDelays[RIGHT] = millis() + DELAY_BETWEEN_LONG_PRESS;
    digitalWrite(PIN_LED, HIGH);
    ledOnTime = millis() + LED_FLASH_TIME;
    SPI_sendIntr((char *)"right");
  }
  
  if (digitalRead(PIN_BTN_UP) == LOW && buttonDelays[UP] < millis()) {
    #ifdef DEBUG
      Serial.println("up");
    #endif
    
    buttonDelays[UP] = millis() + DELAY_BETWEEN_LONG_PRESS;
    digitalWrite(PIN_LED, HIGH);
    ledOnTime = millis() + LED_FLASH_TIME;
    SPI_sendIntr((char *)"up");
  }
  
  if (digitalRead(PIN_BTN_DOWN) == LOW && buttonDelays[DOWN] < millis()) {
    #ifdef DEBUG
      Serial.println("down");
    #endif
    
    buttonDelays[DOWN] = millis() + DELAY_BETWEEN_LONG_PRESS;
    digitalWrite(PIN_LED, HIGH);
    ledOnTime = millis() + LED_FLASH_TIME;
    SPI_sendIntr((char *)"down");
  }
}

void onEndTransimission(void) {
  #ifdef DEBUG
    Serial.println("Send complete");
  #endif
}

/** 
 * Register interrupt handler for SPI 
 * note that this isn't a method declaration but only a macro call
 */
ISR(SPI_STC_vect) {
        // delegate to our SPI helper handler
        // onEndTransmission: callback that will be invoked when we finished transmitting
        SPI_handleSendInterrupt(&onEndTransimission);
}
